from typing import Protocol
from datetime import datetime


class Clock(Protocol):
    def sleep(self, seconds: float) -> None: ...
    def monotonic(self) -> float: ...
    def now_utc(self) -> datetime: ...
